@@ -255,7 +255,7 @@ end
     API.moveBefore(newo, err, B)
     normal =
         (unsafe_load(normalR) != C_NULL) ? LLVM.Instruction(unsafe_load(normalR)) : nothing
-    if shadowR != C_NULL
+    if unsafe_load(shadowR) != C_NULL
         t_shadow1 = if normal !== nothing
             normal
         else
@@ -404,21 +404,39 @@ end
     newg = new_from_original(gutils, orig)
 
     real_ops = collect(operands(orig))[1:end-1]
-    ops = [lookup_value(gutils, new_from_original(gutils, o), B) for o in real_ops]
+    ops = [new_from_original(gutils, o) for o in real_ops]
 
-    c = call_samefunc_with_inverted_bundles!(
+    batch_call_same_with_inverted_arg_if_active!(
         B,
         gutils,
         orig,
-        ops,
-        [API.VT_Primal for _ in ops],
-        false,
-    ) #=lookup=#
-    callconv!(c, callconv(orig))
+        [shadowin],
+        [API.VT_Primal for o in real_ops],
+        true;
+        need_result=false
+    )
 
     return nothing
 end
 
+function post_arraycopy_memset(B, callv, _, _)
+    i8 = LLVM.IntType(8)
+    algn = 0
+
+    elSize = get_array_elsz(B, callv)
+    elSize = LLVM.zext!(B, elSize, LLVM.IntType(8 * sizeof(Csize_t)))
+    len = get_array_len(B, callv)
+    length = LLVM.mul!(B, len, elSize)
+
+    LLVM.memset!(
+        B,
+        get_array_data(B, callv),
+        LLVM.ConstantInt(i8, 0, false),
+        length,
+        algn,
+    )
+end
+
 @register_fwd function arraycopy_fwd(B, orig, gutils, normalR, shadowR)
     ctx = LLVM.context(orig)
 
@@ -432,76 +450,30 @@ end
 
     shadowin = invert_pointer(gutils, origops[1], B)
 
-    i8 = LLVM.IntType(8)
-    algn = 0
-
     shadowres =
         UndefValue(LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(orig))))
 
     found, arty, byref = abs_typeof(origops[1])
 
-    for idx = 1:width
-        ev = if width == 1
-            shadowin
-        else
-            extract_value!(B, shadowin, idx - 1)
-        end
-
-        callv = call_samefunc_with_inverted_bundles!(
-            B,
-            gutils,
-            orig,
-            [ev],
-            [API.VT_Shadow],
-            false,
-        ) #=lookup=#
-        if is_constant_value(gutils, origops[1])
-            elSize = get_array_elsz(B, ev)
-            elSize = LLVM.zext!(B, elSize, LLVM.IntType(8 * sizeof(Csize_t)))
-            len = get_array_len(B, ev)
-            length = LLVM.mul!(B, len, elSize)
-
-            if !found && !(eltype(arty) <: Base.IEEEFloat)
-                bt = GPUCompiler.backtrace(orig)
-                btstr = sprint() do io
-                    print(io, "\nCaused by:")
-                    Base.show_backtrace(io, bt)
-                end
-                GPUCompiler.@safe_warn "TODO forward zero-set of arraycopy used memset rather than runtime type $btstr"
-            end
-
-            LLVM.memset!(
-                B,
-                get_array_data(B, callv),
-                LLVM.ConstantInt(i8, 0, false),
-                length,
-                algn,
-            )
-        end
-        if get_runtime_activity(gutils)
-            prev = new_from_original(gutils, orig)
-            callv = LLVM.select!(
-                B,
-                LLVM.icmp!(
-                    B,
-                    LLVM.API.LLVMIntNE,
-                    ev,
-                    new_from_original(gutils, origops[1]),
-                ),
-                callv,
-                prev,
-            )
-            if idx == 1
-                API.moveBefore(prev, callv, B)
-            end
-        end
-        shadowres = if width == 1
-            callv
-        else
-            insert_value!(B, shadowres, callv, idx - 1)
+    if !found && !(eltype(arty) <: Base.IEEEFloat)
+        bt = GPUCompiler.backtrace(orig)
+        btstr = sprint() do io
+            print(io, "\nCaused by:")
+            Base.show_backtrace(io, bt)
         end
+        GPUCompiler.@safe_warn "TODO forward zero-set of arraycopy used memset rather than runtime type $btstr"
     end
 
+    shadowres = batch_call_same_with_inverted_arg_if_active!(
+        B,
+        gutils,
+        orig,
+        [shadowin],
+        [API.VT_Shadow],
+        false;
+        postprocess_const=post_arraycopy_memset
+    )::LLVM.Value #=lookup=#
+
     unsafe_store!(shadowR, shadowres.ref)
     return false
 end
@@ -746,6 +718,25 @@ end
     return nothing
 end
 
+function post_genericmemcpy_memset(B, callv, args, _)
+    _, _, len = args
+
+    elSize = get_memory_elsz(B, callv)
+    elSize = LLVM.zext!(B, elSize, LLVM.IntType(8 * sizeof(Csize_t)))
+    length = LLVM.mul!(B, len, elSize)
+
+    i8 = LLVM.IntType(8)
+    algn = 0
+
+    LLVM.memset!(
+        B,
+        get_memory_data(B, callv),
+        LLVM.ConstantInt(i8, 0, false),
+        length,
+        algn,
+    )
+end
+
 @register_fwd function genericmemory_copy_slice_fwd(B, orig, gutils, normalR, shadowR)
     ctx = LLVM.context(orig)
 
@@ -761,78 +752,28 @@ end
     shadowdata = invert_pointer(gutils, origops[2], B)
     len = new_from_original(gutils, origops[3])
 
-    i8 = LLVM.IntType(8)
-    algn = 0
 
     found, arty, byref = abs_typeof(origops[1])
 
-    shadowres =
-        UndefValue(LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(orig))))
-    for idx = 1:width
-        ev = if width == 1
-            shadowin
-        else
-            extract_value!(B, shadowin, idx - 1)
-        end
-        ev2 = if width == 1
-            shadowdata
-        else
-            extract_value!(B, shadowdata, idx - 1)
-        end
-        callv = call_samefunc_with_inverted_bundles!(
-            B,
-            gutils,
-            orig,
-            [ev, ev2, len],
-            [API.VT_Shadow, API.VT_Shadow, API.VT_Primal],
-            false,
-        ) #=lookup=#
-        if is_constant_value(gutils, origops[1])
-            elSize = get_memory_elsz(B, ev)
-            elSize = LLVM.zext!(B, elSize, LLVM.IntType(8 * sizeof(Csize_t)))
-            length = LLVM.mul!(B, len, elSize)
-
-            if !found && !(eltype(arty) <: Base.IEEEFloat)
-                bt = GPUCompiler.backtrace(orig)
-                btstr = sprint() do io
-                    print(io, "\nCaused by:")
-                    Base.show_backtrace(io, bt)
-                end
-                GPUCompiler.@safe_warn "TODO forward zero-set of memorycopy used memset rather than runtime type $btstr"
-            end
-
-            LLVM.memset!(
-                B,
-                get_memory_data(B, callv),
-                LLVM.ConstantInt(i8, 0, false),
-                length,
-                algn,
-            )
-        end
-        if get_runtime_activity(gutils)
-            prev = new_from_original(gutils, orig)
-            callv = LLVM.select!(
-                B,
-                LLVM.icmp!(
-                    B,
-                    LLVM.API.LLVMIntNE,
-                    ev,
-                    new_from_original(gutils, origops[1]),
-                ),
-                callv,
-                prev,
-            )
-            if idx == 1
-                API.moveBefore(prev, callv, B)
-            end
-        end
-        shadowres = if width == 1
-            callv
-        else
-            insert_value!(B, shadowres, callv, idx - 1)
+    if !found && !(eltype(arty) <: Base.IEEEFloat)
+        bt = GPUCompiler.backtrace(orig)
+        btstr = sprint() do io
+            print(io, "\nCaused by:")
+            Base.show_backtrace(io, bt)
         end
+        GPUCompiler.@safe_warn "TODO forward zero-set of memorycopy used memset rather than runtime type $btstr"
     end
 
+    shadowres = batch_call_same_with_inverted_arg_if_active!(
+        B,
+        gutils,
+        orig,
+        [shadowin, shadowdata, len],
+        [API.VT_Shadow, API.VT_Shadow, API.VT_Primal],
+        false;
+        postprocess_const=post_genericmemcpy_memset
+    ) #=lookup=#
+
     unsafe_store!(shadowR, shadowres.ref)
     return false
 end
@@ -956,46 +897,28 @@ end
     if unsafe_load(shadowR) != C_NULL
 	    origops = LLVM.operands(orig)
 	    if is_constant_value(gutils, origops[2])
-		emit_error(B, orig, "Enzyme: reshape array has active return, but inactive input")
+    		emit_error(B, orig, "Enzyme: reshape array has active return, but inactive input")
 	    end
 
 	    width = get_width(gutils)
 
 	    shadowin = invert_pointer(gutils, origops[2], B)
-	    if width == 1
-		args = LLVM.Value[
+
+	    args = LLVM.Value[
 		    new_from_original(gutils, origops[1])
-		    shadowin
-		    new_from_original(gutils, origops[3])
-		]
-		shadowres = call_samefunc_with_inverted_bundles!(
-		    B,
-		    gutils,
-		    orig,
-		    args,
-		    [API.VT_Primal, API.VT_Shadow, API.VT_Primal],
-		    false,
-		) #=lookup=#
-	    else
-		shadowres =
-		    UndefValue(LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(orig))))
-		for idx = 1:width
-		    args = LLVM.Value[
-			new_from_original(gutils, origops[1])
-			extract_value!(B, shadowin, idx - 1)
-			new_from_original(gutils, origops[3])
-		    ]
-		    tmp = call_samefunc_with_inverted_bundles!(
+            shadowin
+            new_from_original(gutils, origops[3])
+	    ]
+
+		shadowres = batch_call_same_with_inverted_arg_if_active!(
 			B,
 			gutils,
 			orig,
 			args,
 			[API.VT_Primal, API.VT_Shadow, API.VT_Primal],
-			false,
+			false;
+            cmpidx = 2
 		    ) #=lookup=#
-		    shadowres = insert_value!(B, shadowres, tmp, idx - 1)
-		end
-	    end
 	    unsafe_store!(shadowR, shadowres.ref)
     end
     return false
@@ -1036,35 +959,19 @@ end
 
     shadowin1 = invert_pointer(gutils, origops[1], B)
     shadowin2 = invert_pointer(gutils, origops[2], B)
-    if width == 1
-        args = LLVM.Value[shadowin1, shadowin2]
-        shadowres = call_samefunc_with_inverted_bundles!(
-            B,
-            gutils,
-            orig,
-            args,
-            [API.VT_Shadow, API.VT_Shadow],
-            false,
-        ) #=lookup=#
-    else
-        shadowres =
-            UndefValue(LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(orig))))
-        for idx = 1:width
-            args = LLVM.Value[
-                extract_value!(B, shadowin1, idx - 1)
-                extract_value!(B, shadowin2, idx - 1)
-            ]
-            tmp = call_samefunc_with_inverted_bundles!(
-                B,
-                gutils,
-                orig,
-                args,
-                [API.VT_Shadow, API.VT_Shadow],
-                false,
-            ) #=lookup=#
-            shadowres = insert_value!(B, shadowres, tmp, idx - 1)
-        end
-    end
+    args = LLVM.Value[
+        shadowin1,
+        shadowin2
+    ]
+    shadowres = batch_call_same_with_inverted_arg_if_active!(
+        B,
+        gutils,
+        orig,
+        args,
+        [API.VT_Shadow, API.VT_Shadow],
+        false,
+    ) #=lookup=#
+
     unsafe_store!(shadowR, shadowres.ref)
 
     return false
@@ -1325,42 +1232,27 @@ end
 
     newvals = API.CValueType[API.VT_Shadow, API.VT_Primal, API.VT_Shadow]
 
-    shadowres = if width == 1
-        newops = LLVM.Value[shadowh, new_from_original(gutils, origkey), shadowdflt]
-        cal = call_samefunc_with_inverted_bundles!(B, gutils, orig, newops, newvals, false) #=lookup=#
-        callconv!(cal, callconv(orig))
+    function post_err_if_active(B, cal, args)
         emit_apply_generic!(
             B,
             LLVM.Value[unsafe_to_llvm(B, error_if_active), emit_jltypeof!(B, cal)],
         )
-        cal
-    else
-        ST = LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(orig)))
-        shadow = LLVM.UndefValue(ST)
-        for j = 1:width
-            newops = LLVM.Value[
-                extract_value!(B, shadowh, j - 1),
-                new_from_original(gutils, origkey),
-                extract_value!(B, shadowdflt, j - 1),
-            ]
-            cal = call_samefunc_with_inverted_bundles!(
-                B,
-                gutils,
-                orig,
-                newops,
-                newvals,
-                false,
-            ) #=lookup=#
-            callconv!(cal, callconv(orig))
-            emit_apply_generic!(
-                B,
-                LLVM.Value[unsafe_to_llvm(B, error_if_active), emit_jltypeof!(B, cal)],
-            )
-            shadow = insert_value!(B, shadow, cal, j - 1)
-        end
-        shadow
     end
 
+    newops = LLVM.Value[
+        shadowh,
+        new_from_original(gutils, origkey),
+        shadowdflt,
+    ]
+    shadowres = batch_call_same_with_inverted_arg_if_active!(
+        B,
+        gutils,
+        orig,
+        newops,
+        newvals,
+        false,
+    )
+
     unsafe_store!(shadowR, shadowres.ref)
     return false
 end
@@ -1395,6 +1287,15 @@ end
     return false
 end
 
+function eqtable_shadow_active(B, args)
+    _, _, shadowval, _ = args
+    emit_apply_generic!(
+        B,
+        LLVM.Value[unsafe_to_llvm(B, error_if_active), emit_jltypeof!(B, shadowval)],
+    )
+    return nothing
+end
+
 @register_aug function eqtableput_augfwd(B, orig, gutils, normalR, shadowR, tapeR)
     if is_constant_value(gutils, orig) && is_constant_inst(gutils, orig)
         return true
@@ -1447,48 +1348,15 @@ end
 
     newvals = API.CValueType[API.VT_Shadow, API.VT_Primal, API.VT_Shadow, API.VT_None]
 
-    shadowres = if width == 1
-        emit_apply_generic!(
-            B,
-            LLVM.Value[unsafe_to_llvm(B, error_if_active), emit_jltypeof!(B, shadowval)],
-        )
-        newops = LLVM.Value[
-            shadowh,
-            new_from_original(gutils, origkey),
-            shadowval,
-            LLVM.null(value_type(originserted)),
-        ]
-        cal = call_samefunc_with_inverted_bundles!(B, gutils, orig, newops, newvals, false) #=lookup=#
-        callconv!(cal, callconv(orig))
-        cal
-    else
-        ST = LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(orig)))
-        shadow = LLVM.UndefValue(ST)
-        for j = 1:width
-            sval2 = extract_value!(B, shadowval, j - 1)
-            emit_apply_generic!(
-                B,
-                LLVM.Value[unsafe_to_llvm(B, error_if_active), emit_jltypeof!(B, sval2)],
-            )
-            newops = LLVM.Value[
-                extract_value!(B, shadowh, j - 1),
-                new_from_original(gutils, origkey),
-                sval2,
-                LLVM.null(value_type(originserted)),
-            ]
-            cal = call_samefunc_with_inverted_bundles!(
-                B,
-                gutils,
-                orig,
-                newops,
-                newvals,
-                false,
-            ) #=lookup=#
-            callconv!(cal, callconv(orig))
-            shadow = insert_value!(B, shadow, cal, j - 1)
-        end
-        shadow
-    end
+    newops = LLVM.Value[
+      shadowh,
+      new_from_original(gutils, origkey),
+      shadowval,
+      LLVM.null(value_type(originserted)),
+    ]
+
+    shadowres = batch_call_same_with_inverted_arg_if_active!(B, gutils, orig, newops, newvals, false;
+        preprocess=eqtable_shadow_active) #=lookup=#
 
     unsafe_store!(shadowR, shadowres.ref)
     return false
@@ -1570,38 +1438,47 @@ end
     width = get_width(gutils)
 
     shadowin = invert_pointer(gutils, origops[1], B)
-    if width == 1
-        args = LLVM.Value[
-            shadowin
-            new_from_original(gutils, origops[2])
-        ]
-        call_samefunc_with_inverted_bundles!(
-            B,
-            gutils,
-            orig,
-            args,
-            [API.VT_Shadow, API.VT_Primal],
-            false,
-        ) #=lookup=#
-    else
-        for idx = 1:width
-            args = LLVM.Value[
-                extract_value!(B, shadowin, idx - 1)
-                new_from_original(gutils, origops[2])
-            ]
-            call_samefunc_with_inverted_bundles!(
-                B,
-                gutils,
-                orig,
-                args,
-                [API.VT_Shadow, API.VT_Primal],
-                false,
-            ) #=lookup=#
-        end
-    end
+    args = LLVM.Value[
+        shadowin
+        new_from_original(gutils, origops[2])
+    ]
+    batch_call_same_with_inverted_arg_if_active!(
+        B,
+        gutils,
+        orig,
+        args,
+        [API.VT_Shadow, API.VT_Primal],
+        false;
+        need_result = false
+    ) #=lookup=#
     return false
 end
 
+function pre_shadow_array_grow!(B, args)
+    anti, inc = args
+
+    idx = get_array_nrows(B, anti)
+
+    return idx
+end
+
+function post_shadow_array_grow!(B, _, args, pre)
+    al = 0
+    anti, inc = args
+
+    idx = pre
+
+    i8 = LLVM.IntType(8)
+
+    elsz = zext!(B, get_array_elsz(B, anti), value_type(idx))
+    off = mul!(B, idx, elsz)
+    tot = mul!(B, inc, elsz)
+
+    toset = get_array_data(B, anti)
+    toset = gep!(B, i8, toset, LLVM.Value[off])
+    mcall = LLVM.memset!(B, toset, LLVM.ConstantInt(i8, 0, false), tot, al)
+
+end
 
 @register_aug function jl_array_grow_end_augfwd(B, orig, gutils, normalR, shadowR, tapeR)
     origops = collect(operands(orig))
@@ -1613,57 +1490,23 @@ end
 
     shadowin = invert_pointer(gutils, origops[1], B)
     ctx = LLVM.context(orig)
-    i8 = LLVM.IntType(8)
+
 
     inc = new_from_original(gutils, origops[2])
 
-    al = 0
+    args = LLVM.Value[shadowin, inc]
+    batch_call_same_with_inverted_arg_if_active!(
+        B,
+        gutils,
+        orig,
+        args,
+        [API.VT_Shadow, API.VT_Primal],
+        false;
+        preprocess=pre_shadow_array_grow!,
+        postprocess=post_shadow_array_grow!,
+        need_result = false
+    ) #=lookup=#
 
-    if width == 1
-        anti = shadowin
-
-        idx = get_array_nrows(B, anti)
-        elsz = zext!(B, get_array_elsz(B, anti), value_type(idx))
-        off = mul!(B, idx, elsz)
-        tot = mul!(B, inc, elsz)
-
-        args = LLVM.Value[anti, inc]
-        call_samefunc_with_inverted_bundles!(
-            B,
-            gutils,
-            orig,
-            args,
-            [API.VT_Shadow, API.VT_Primal],
-            false,
-        ) #=lookup=#
-
-        toset = get_array_data(B, anti)
-        toset = gep!(B, i8, toset, LLVM.Value[off])
-        mcall = LLVM.memset!(B, toset, LLVM.ConstantInt(i8, 0, false), tot, al)
-    else
-        for idx = 1:width
-            anti = extract_value!(B, shadowin, idx - 1)
-
-            idx = get_array_nrows(B, anti)
-            elsz = zext!(B, get_array_elsz(B, anti), value_type(idx))
-            off = mul!(B, idx, elsz)
-            tot = mul!(B, inc, elsz)
-
-            args = LLVM.Value[anti, inc]
-            call_samefunc_with_inverted_bundles!(
-                B,
-                gutils,
-                orig,
-                args,
-                [API.VT_Shadow, API.VT_Primal],
-                false,
-            ) #=lookup=#
-
-            toset = get_array_data(B, anti)
-            toset = gep!(B, i8, toset, LLVM.Value[off])
-            mcall = LLVM.memset!(B, toset, LLVM.ConstantInt(i8, 0, false), tot, al)
-        end
-    end
 
     return false
 end
@@ -1685,21 +1528,45 @@ end
         offset = new_from_original(gutils, origops[2])
         offset = lookup_value(gutils, offset, B)
 
-        if width == 1
+        currentBlock = Base.position(B)
+        ogname = LLVM.name(currentBlock)
+
+        fval = if get_runtime_activity(gutils)
+            lookup_value(gutils, new_from_original(gutils, origops[1]), B)
+        else
+            nothing
+        end
+
+        endB = nothing
+
+        for idx = 1:width
             args = LLVM.Value[
                 shadowin
                 offset
             ]
+            if width > 1
+                extract_value!(B, shadowin, idx - 1)
+            end
+
+            if get_runtime_activity(gutils)
+                cond = icmp!(B, LLVM.API.LLVMIntNE, fval, args[1])
+
+                nextB = add_reverse_block!(gutils, currentBlock, ogname*"_active")
+
+                endB = add_reverse_block!(gutils, nextB, ogname*"_end", true, false)
+
+                br!(B, cond, nextB, endB)
+
+                position!(B, nextB)
+            end
+
             LLVM.call!(B, fty, delF, args)
-        else
-            shadowres =
-                UndefValue(LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(orig))))
-            for idx = 1:width
-                args = LLVM.Value[
-                    extract_value!(B, shadowin, idx - 1)
-                    offset
-                ]
-                LLVM.call!(B, fty, delF, args)
+
+            if get_runtime_activity(gutils)
+                br!(B, endB)
+                set_reverse_block!(gutils, endB)
+                position!(B, endB)
+                currentBlock = endB
             end
         end
     end
@@ -1830,26 +1697,8 @@ end
         API.VT_Primal,
     ]
 
-    if width == 1
-        vargs = args
-        cal = call_samefunc_with_inverted_bundles!(B, gutils, orig, vargs, valTys, false) #=lookup=#
-        debug_from_orig!(gutils, cal, orig)
-        callconv!(cal, callconv(orig))
-    else
-        shadowres =
-            UndefValue(LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(orig))))
-        for idx = 1:width
-            vargs = LLVM.Value[]
-            for a in args[1:end-1]
-                push!(vargs, extract_value!(B, a, idx - 1))
-            end
-            push!(vargs, args[end])
-            cal =
-                call_samefunc_with_inverted_bundles!(B, gutils, orig, vargs, valTys, false) #=lookup=#
-            debug_from_orig!(gutils, cal, orig)
-            callconv!(cal, callconv(orig))
-        end
-    end
+    batch_call_same_with_inverted_arg_if_active!(B, gutils, orig, args, valTys, false;
+            need_result = false) #=lookup=#
 
     return false
 end
@@ -1875,35 +1724,20 @@ end
         API.VT_Primal,
     ]
 
-    shadowres = UndefValue(LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(orig))))
-    for idx = 1:width
-        ev = if width == 1
-            shadowin
-        else
-            extract_value!(B, shadowin, idx - 1)
-        end
-
 	args = LLVM.Value[
 	        new_from_original(gutils, origops[1]),
-	        ev, # data
+	        shadowin, # data
 	        new_from_original(gutils, origops[3]),
 	        new_from_original(gutils, origops[4]),
 	]
-	# TODO do runtime activity relevant errors and checks
 
-        cal = call_samefunc_with_inverted_bundles!(B, gutils, orig, args, valTys, false) #=lookup=#
-        debug_from_orig!(gutils, cal, orig)
-        callconv!(cal, callconv(orig))
-	if width == 1
-	    shadowres = cal
-	else
-            shadowres = insert_value!(B, shadowres, call, idx - 1)
-	end
-    end
+    shadowres = batch_call_same_with_inverted_arg_if_active!(B, gutils, orig, args, valTys, false; cmpidx=2)::LLVM.Value
+
     unsafe_store!(shadowR, shadowres.ref)
 
     return false
 end
+
 @register_aug function jl_ptr_to_array_augfwd(B, orig, gutils, normalR, shadowR, tapeR)
     jl_ptr_to_array_fwd(B, orig, gutils, normalR, shadowR)
 end
