@@ -462,10 +462,23 @@ end
 
 else
 
+function is_memory_ref_field2_an_offset(@nospecialize(T::Type{<:GenericMemoryRef}))
+    ET = eltype(T)
+
+    # 0 = inlinealloc
+    # 1 = isboxed
+    # 2 = isbitsunion
+    return (Base.datatype_arrayelem(T.types[2]) == 2) || Base.datatype_layoutsize(T.types[2]) == 0
+end
+
 @inline function typed_fieldtype(@nospecialize(T::Type), i::Int)::Type
     if T <: GenericMemoryRef && i == 1 || T <: GenericMemory && i == 2
-        eT = eltype(T)
-        Ptr{eT}
+        if T <: GenericMemoryRef && i == 1 && is_memory_ref_field2_an_offset(T)
+            Int
+        else
+            eT = eltype(T)
+            Ptr{eT}
+        end
     else
         fieldtype(T, i)
     end
